<?php

namespace loyalsoft;

ini_set("display_errors", "on");                                                # 打开错误输出
require_once __DIR__ . '/msmapi/api_sdk/vendor/autoload.php';

use Aliyun\Core\Config;
use Aliyun\Core\Profile\DefaultProfile;
use Aliyun\Core\DefaultAcsClient;
use Aliyun\Api\Sms\Request\V20170525\SendSmsRequest;
use Aliyun\Api\Sms\Request\V20170525\QuerySendDetailsRequest;

Config::load();                                                                 // 加载区域结点配置

/**
 * 阿里云 - 短信Util
 *
 * @property \Aliyun\Core\DefaultAcsClient acsClient
 */
class SmsUtil {

    /**
     * asc(http)网络通信
     * @var \Aliyun\Core\DefaultAcsClient 阿里
     */
    var $acsClient;

    /**
     * 单例模式
     * @staticvar type $smsObj
     * @return \SmsObject
     */
    private static function inst() {
        static $smsObj = null;
        if (is_null($smsObj)) {
            $smsObj = new SmsUtil(
                    "FtwrIeI6uXLylJ0h", "SAI0ZJf7qrvngvkZCU6qgQF3d7A4dP"
            );
        }
        return $smsObj;
    }

    /**
     * 构造器
     *
     * @param string $accessKeyId 必填，AccessKeyId
     * @param string $accessKeySecret 必填，AccessKeySecret
     */
    private function __construct($accessKeyId, $accessKeySecret) {
        $product = "Dysmsapi";                                                  // 短信API产品名
        $domain = "dysmsapi.aliyuncs.com";                                      // 短信API产品域名
        $region = "cn-hangzhou";                                                // 暂时不支持多Region
        $endPointName = "cn-hangzhou";                                          // 服务结点
        $profile = DefaultProfile::getProfile($region, $accessKeyId, $accessKeySecret); // 初始化用户Profile实例
        DefaultProfile::addEndpoint($endPointName, $region, $product, $domain); // 增加服务结点
        $this->acsClient = new DefaultAcsClient($profile);                      // 初始化AcsClient用于发起请求
    }

    /**
     * 发送短信范例
     *
     * @param string $signName <p>
     * 必填, 短信签名，应严格"签名名称"填写，参考：<a href="https://dysms.console.aliyun.com/dysms.htm#/sign">短信签名页</a>
     * </p>
     * @param string $templateCode <p>
     * 必填, 短信模板Code，应严格按"模板CODE"填写, 参考：<a href="https://dysms.console.aliyun.com/dysms.htm#/template">短信模板页</a>
     * (e.g. SMS_0001)
     * </p>
     * @param string $phoneNumbers 必填, 短信接收号码 (e.g. 12345678901)
     * @param array|null $templateParam <p>
     * 选填, 假如模板中存在变量需要替换则为必填项 (e.g. Array("code"=>"12345", "product"=>"阿里通信"))
     * </p>
     * @param string|null $outId [optional] 选填, 发送短信流水号 (e.g. 1234)
     * @return stdClass
     */
    static public function sendSms($signName, $templateCode, $phoneNumbers, $templateParam = null, $outId = null) {
        $request = new SendSmsRequest();                                        // 初始化SendSmsRequest实例用于设置发送短信的参数
        $request->setPhoneNumbers($phoneNumbers);                               // 必填，设置雉短信接收号码
        $request->setSignName($signName);                                       // 必填，设置签名名称
        $request->setTemplateCode($templateCode);                               // 必填，设置模板CODE
        if ($templateParam) {                                                   // 可选，设置模板参数
            $request->setTemplateParam(json_encode($templateParam));
        }
        if ($outId) {                                                           // 可选，设置流水号
            $request->setOutId($outId);
        }
        $acsResponse = self::inst()->acsClient->getAcsResponse($request);       // 发起访问请求
        // var_dump($acsResponse);                                              // 打印请求结果
        return $acsResponse;
    }

    /**
     * 查询短信发送情况范例
     *
     * @param string $phoneNumbers 必填, 短信接收号码 (e.g. 12345678901)
     * @param string $sendDate 必填，短信发送日期，格式Ymd，支持近30天记录查询 (e.g. 20170710)
     * @param int $pageSize 必填，分页大小
     * @param int $currentPage 必填，当前页码
     * @param string $bizId 选填，短信发送流水号 (e.g. abc123)
     * @return stdClass
     */
    static public function queryDetails($phoneNumbers, $sendDate, $pageSize = 10, $currentPage = 1, $bizId = null) {
        $request = new QuerySendDetailsRequest();                               // 初始化QuerySendDetailsRequest实例用于设置短信查询的参数
        $request->setPhoneNumber($phoneNumbers);                                // 必填，短信接收号码
        $request->setBizId($bizId);                                             // 选填，短信发送流水号
        $request->setSendDate($sendDate);                                       // 必填，短信发送日期，支持近30天记录查询，格式Ymd
        $request->setPageSize($pageSize);                                       // 必填，分页大小
        $request->setCurrentPage($currentPage);                                 // 必填，当前页码
        $acsResponse = self::inst()->acsClient->getAcsResponse($request);       // 发起访问请求
        // var_dump($acsResponse);                                              // 打印请求结果
        return $acsResponse;
    }

}
